#
# IBM Confidential
# OCO Source Materials
# Rational ClearQuest
# (C) Copyright IBM Corp. 2004, 2008.  All Rights Reserved.
#
# The source code for this program is not published or otherwise
# divested of its trade secrets, irrespective of what has been
# deposited with the U.S. Copyright Office.
#
# Description
#
# This utility will list all the non LDAP authenticated (i.e. CQ Authenticated) 
# users for the database and schema repository specified. The listed users 
# will need to be LDAP authenticated to be FIPS140-2 compliant.
# It will also print the feature level of the database. If the FL is 7 then 
# database is FIPS 140-2 compliant, i.e. the passwords in the master_dbs 
# and master_property tables have been updated with FIPS140-2 compliant software
#
################################################################################
# Usage:
#
# ratlperl FIPSreport.pl
#    [-HELP] 
#    [-cmd {nonldapusers|feature_level}]
#    [-CQADMINUSER <CQ admin user>]
#    [-CQADMINPASSWORD <CQ admin pswd>]
#    [-DBSET <dbset>]
#    [-DB <db>]
#
################################################################################


use CQPerlExt;

use strict;
use warnings;

# Define default values...
my(%ARG) = (
    CQADMINUSER => "admin",
    CQADMINPASSWORD => "",
);

# Define command syntax...
my($Usage) = "Usage:\n"
    ."\n"
    ."  cqperl FIPSReport.pl  [-HELP] |\n"
    ."                        [-CMD {nonldapusers|feature_level}]\n"
    ."                        [-CQADMINUSER <CQ admin user>]\n"
    ."                        [-CQADMINPASSWORD <CQ admin pwd>]\n"
    ."                        [-DBSET <dbset>]\n"
    ."                        [-DB <db>]\n"
    ."\n"
    ."  Parameters:\n"
    ."    -HELP             Displays this usage message\n"
    ."    -CMD              One of \"nonldapusers\" or \"feature_level\" to display either the Non-LDAP users or the feature level of the db respectively\n"
    ."    -DBSET:           CQ DBSet (connection) name\n"
    ."    -CQADMINUSER:     CQ administrative user (default='$ARG{CQADMINUSER}')\n"
    ."    -CQADMINPASSWORD: CQ administrative user's password (default='$ARG{CQADMINPASSWORD}')\n"
    ."    -DB:              CQ database whose feature level should be shown. If this is not specified, feature levels of all DBs are shown\n"
;

# Parse the user's command-line parameters...
while (@ARGV) {
    my($A) = shift;
    if ($A =~ /-(CMD|DBSET|CQADMINUSER|CQADMINPASSWORD|DB)/i) {
        # Keyword/value parameter; get the value...
        $ARG{uc $1} = shift;
    } elsif ($A =~ /-(HELP)/i) {
        print $Usage;
        exit;
    } else {
        die "ERROR: Unrecognized argument '$A'.";
    }
}

my(@Errors);
push @Errors, "ERROR: Missing parameter 'CMD'" if (not $ARG{CMD});
push @Errors, "Invalid value for CMD parameter" if (not($ARG{CMD} =~ /NONLDAPUSERS|FEATURE_LEVEL/i));
push @Errors, "ERROR: Missing parameter 'CQADMINUSER'" if (not $ARG{CQADMINUSER});
push @Errors, "ERROR: Missing parameter 'DBSET'" if (not $ARG{DBSET});
if (@Errors) {
    print join("\n", @Errors)
        ."\n\n$Usage"
    ;
    exit 1;
}


###############################################################################

my $AdminSession = CQAdminSession::Build();

$AdminSession->Logon($ARG{CQADMINUSER}, $ARG{CQADMINPASSWORD}, $ARG{DBSET});
if ($@){print "Error in logon: for $ARG{CQADMINUSER} $ARG{CQADMINPASSWORD} $ARG{DB} $ARG{DBSET} \n$@\n";die;}

if ($ARG{CMD} =~ /nonldapusers/i) {

    ###########################
    #Get the list of users in the repository.
    my $userList = $AdminSession->GetUsers();

    #Get the number of users
    my $numUsers = $userList->Count();

    #// FYI:-------------------  AuthenticationMode  ------------------- //
    #// These have to be in the special order.
    #static const AdEnumStruct AdAuthenticationModeTable[] = {
    #	{ _T("LDAP_AUTHENTICATION"),1 },
    #	{ _T("CQ_AUTHENTICATION"),2 },
    #	{ _T("LDAP_AUTHENTICATION"),1 },
    #	{ _T("CQ_AUTHENTICATION"),2 },
    #	{	  NULL, 0 },
    #};


#######################################################################################
# Print users that are not FIPS140-2 compliant, so list all non LDAP_AUTHENTICATION users

    print "A necessary condition for FIPS 140-2 configuration is for user account to be LDAP_AUTHENTICATED\n";
    print "The following are user accounts in $ARG{DBSET} that are not LDAP_AUTHENTICATED.\n";
    print "<username>,<full name>,<email>\n";

    #Iterate through the users
    for ( my $x=0; $x<$numUsers; $x++ )
    {
    	#Get the specified item in the collection of users
    	my $userObj = $userList->Item( $x );

    	#Get the name of the user
    	my $userName = $userObj->GetName();
        my $fullName = $userObj->GetFullName();
        my $email = $userObj->GetEmail();
    	my $enumval  = $userObj->GetAuthenticationMode();
    	#print "userName is ".$userName."\n";

    	if ( $enumval eq 2)
    	{
    		# Should print these users
    		print "$userName,$fullName,$email\n";
    	}
    }

# End of Print users that are not FIPS140-2 compliant, so list all non LDAP_AUTHENTICATION users
#######################################################################################

}

elsif ($ARG{CMD} =~ /feature_level/i) {

#######################################################################################
# Print the feature level of the database.If not FL7 then not FIPS140-2 compliant.
# FL7 upgrade will update both master_dbs and masper_property tables with new encryption.
# So by the virtue of FL7 implies they are compliant. 

    # If the -DB <db anme> is specified, then list the feature level info only for that db. Otherwise,
    # get all the accessible databases and display the feature level for each of them.
    
    if (not $ARG{DB}) {     # If DB is not specified

        #Get a list of accessible databases
        my $databases = $AdminSession->GetDatabases();
        my $count = $databases->Count();

        print "\n";
        print "Number of databases are ".$count. "\n\n";

        # Login to each database successively.
        for(my $x=0;$x<$count;$x++)
        {
            my $db = $databases->Item($x);
            my $dbName = $db->GetName();
            print "Database name is ".$dbName. "\n";

            my $flevel = $db->GetDatabaseFeatureLevel();
            print "Feature Level is ".$flevel."\n";
        }
    }
    else {      # -DB parameter is specified. List only specific db

        my $db = $AdminSession->GetDatabase($ARG{DB});
        my $dbName = $db->GetName();
        print "Database name is ".$dbName. "\n";
        my $flevel = $db->GetDatabaseFeatureLevel();
        print "Feature Level is ".$flevel."\n";
    }

}

#############################################

CQAdminSession::Unbuild($AdminSession);

#### END ###########



